@@ -1,6 +1,6 @@
 import numpy as np
 from typing import Union
-from math import floor, inf, isclose, ceil
+from math import inf, isclose, ceil
 
 def _check_type(types, *vars):
     for v in vars:
@@ -8,14 +8,53 @@ def _check_type(types, *vars):
             raise TypeError(f"{type(v)} is not of type(s) {types}")
 
 class Series:
+    '''Series object represents ordered one-dimensional data.\n
+    `y` property stores data (of any type accepted by `np.ndarray`), while `x` property stores equidistant, ascending
+    ordering values (e.g. time or frequency).
+    It supports `+`, `-`, `*`, `/`, `+=`, `-=`, `*=`, `/=` operators with some restrictions:
+    - Series object must be left-hand side (lhs)
+    - On right-hand side (rhs) either another `Series` object can be used
+    or any type that is supported by underlying numpy array (eg. another numpy array or a single number)
+    - If operation is performed on two `Series` objects, their x arrays must match,
+    if numpy array is on rhs, its size must match with lhs `Series` object y array size
 
-    def __init__(self, y: np.ndarray, x: Union[np.ndarray, tuple, float] = None, freq = None):
+    Notes
+    -----
+    When new `Series` object is constructed, either explicitly with `Series()`
+    or implicitly, e.g. when performing arithmetical operations or using functions
+    like `slice`, `split`, only array references are copied if possible.
+    Thus, if you modify x or y array in-place, changes may affect other objects.
+    If you want to modify underlying arrays in-place, it's advised to make an explicit copy,
+    either of both arrays with `copy` method or only of y array with `copy_y`
+
+    See also
+    --------
+    `labpy.dsp`:
+        collection of functions for processing data stored in `Series` object
+    '''
+
+    def __init__(self, y: np.ndarray, x: Union[np.ndarray, float] = None, freq: float = None, x0: float = 0.):
+        '''Initialize Series instance. Numpy arrays passed are shallow-copied.
+
+        Parameters
+        ----------
+        y: np.ndarray | Series-like
+            Array with values or Series-like (with `x` and `y` attributes) object.
+        x: np.ndarray | float
+            Array of equidistant ordering values (e.g. time or frequency)
+            or single value equal to length of measurement (e.g. total time). Exclusive with `freq` parameter.
+        freq: float
+            Frequency of samples. Exclusive with `x` parameter.
+        x0: float
+            Value added to x array when it's not passed explicitly,
+            i.e. is defined by `freq` or single-valued `x` parameter. Ignored otherwise.
+        '''
         if x is None and freq is None:
-            self._x = np.asarray(y.x)
-            self._y = np.asarray(y.y)
-        else:
-            self._x: np.ndarray = Series.calc_x(y, x, freq)
-            self._y: np.ndarray = np.asarray(y)
+            # Assume x is Series-like and try to copy attributes
+            x = np.asarray(y.x)
+            y = np.asarray(y.y)
+        self._x: np.ndarray = Series.calc_x(y, x, freq, x0)
+        self._y: np.ndarray = np.asarray(y)
 
     def copy(self):
         return Series(self._y.copy(), self._x.copy())
@@ -73,6 +112,13 @@ class Series:
         return Series(fun(self._x), self._x)
 
     def slice(self, l=-inf, r=inf, rel=False):
+        '''Return new `Series` object constructed by taking view of `x` array
+        for values $l \\leq x < r$ and corrresponding view of `y` array.
+        If `rel = False` (default), $l$ and $r$ are absolute.
+        If `rel = True`, $l$ is relative to `x` array beginning if $l \\geq 0$ and end if $l < 0$,
+        same for $r$ except it points to `x` array end if $r = 0$\n
+        Use `cut` for default relative specification.
+        '''
         rng = self.range
         if rel:
             l = rng[0] + l if l >= 0. else rng[1] + l + self.dx
@@ -82,6 +128,7 @@ class Series:
         return Series(self._y[i0:i1], self._x[i0:i1])
 
     def cut(self, l=-inf, r=inf):
+        '''Same as `slice` with `rel = True`'''
         return self.slice(l, r, rel=True)
 
     def extend(self, l=None, r=None, rel=False):
@@ -91,12 +138,12 @@ class Series:
         if l is not None:
             if rel: l = l0 + l
             li = min(Series.find_idx(l, rng, norm=False), 0)
-            lx = np.linspace(l0 + li * self.dx, l0, -li, endpoint=False)
+            lx = np.linspace(l0 + li * (r0 - l0) / s, l0, -li, endpoint=False)
         else: lx = np.array([])
         if r is not None:
             if rel: r = r0 + r
             ri = max(Series.find_idx(r, rng, norm=False), s)
-            rx = np.linspace(r0, self.dx * (ri - s), (ri - s), endpoint=False)
+            rx = np.linspace(r0, r0 + (ri - s) * (r0 - l0) / s, (ri - s), endpoint=False)
         else: rx = np.array([])
         return Series(
             np.concatenate([np.zeros_like(lx), self._y, np.zeros_like(rx)]),
@@ -146,28 +193,32 @@ class Series:
         return ceil(i)
 
     @staticmethod
-    def from2darray(y2d: np.ndarray, *args):
+    def from2darray(y2d: np.ndarray, *args, **kwargs):
         if(y2d.ndim != 2):
             raise ValueError("y2d array should be two-dimensional")
         nrow = y2d.shape[0]
-        x = Series.calc_x(y2d[0], *args)
+        x = Series.calc_x(y2d[0], *args, **kwargs)
         return [Series(y2d[i], x) for i in range(0, nrow)]
 
     @staticmethod
-    def calc_x(y: np.ndarray, x: Union[np.ndarray, tuple, float], freq = None):
+    def calc_x(y: np.ndarray, x: Union[np.ndarray, float] = None, freq=None, x0 = 0.):
         _check_type(np.ndarray, y)
+        if y.ndim != 1:
+            raise ValueError(f"Array y (dim={y.ndim}), should be one-dimensional")
+        if x is not None and freq is not None:
+            raise ValueError("Either x or freq should be specified, not both")
+        if x is None and freq is None:
+            raise ValueError("Either x or freq should be specified")
+        if freq is not None:
+            x = y.size / freq
         if not isinstance(x, np.ndarray):
-            if freq:
-                if x is None:
-                    x = 0.
-                l, r = x, x + (y.size / freq)
-            else:
-                l, r = x
+            l, r = x0, x0 + x
             x = np.linspace(l, r, y.size, endpoint=False)
-        if x.ndim != 1 or y.ndim != 1:
-            raise ValueError(f"Arrays x (dim={x.ndim}), y (dim={y.ndim}) should be one-dimensional")
-        if(x.size != y.size):
-            raise ValueError(f"Arrays x (size={x.size}), y (size={y.size}) should be of equal size")
+        else:
+            if x.ndim != 1:
+                raise ValueError(f"Array x (dim={x.ndim}), should be one-dimensional")
+            if(x.size != y.size):
+                raise ValueError(f"Array x size = {x.size} should be equal to array y size = {y.size}")
         return x
 
 def _gen_op(op):
@@ -180,23 +231,3 @@ def _gen_np_apply(fun):
     return lambda self: Series(getattr(np, fun)(self._y), self._x)
 for fun in ["abs", "real", "imag", "angle"]:
     setattr(Series, fun, _gen_np_apply(fun))
-
-class Average:
-
-    def __init__(self, v = None):
-        if v is not None:
-            self.sum = v * 1.
-        self.count = 0 if v is None else 1
-
-    def add(self, v):
-        if self.count == 0:
-            self.sum = v * 1.
-        else:
-            self.sum += v
-        self.count += 1
-
-    @property
-    def value(self):
-        if self.count == 0:
-            return 0.
-        return self.sum / self.count
\ No newline at end of file
